from django.shortcuts import render, redirect, get_object_or_404
from django.urls import reverse
from django.views.generic import View, TemplateView, RedirectView
from web.forms import ArticleForm, ArticleModelForm
from web.models import Article, StatusChoices


def index_view(request):
    return render(
        request, 'index.html', context={
            'articles': Article.objects.order_by('-created_at'),
            'reverse': reverse('articles-detail', kwargs={'id': 1})
        }
    )


class MainPageRedirectView(RedirectView):
    pattern_name = 'main_page'


# Class-based views
class ArticleCreateView(View):
    def get(self, request, *args, **kwargs):
        form = ArticleModelForm()
        
        return render(
            self.request,
            'article_create.html',
            context={
                'status_choices': StatusChoices.choices,
                'form': form
            }
        )

    def post(self, request, *args, **kwargs):
        errors = {}
        data = request.POST
        form = ArticleModelForm(data=data)
        if form.is_valid():
            article = form.save()
            return redirect('articles-detail', id=article.id)
        else:
            return render(self.request, 'article_create.html', context={'form': form, 'status_choices': StatusChoices.choices})


# /articles/{id}
def article_detail_view(request, id: int):
    article = get_object_or_404(Article, id=id)
    return render(request, 'article_detail.html', context={'article': article})


class ArticleDetailView(TemplateView):
    template_name = 'article_detail.html'

    def get_context_data(self, **kwargs):
        article = get_object_or_404(Article, id=kwargs.get('id'))
        return super().get_context_data(article=article)

def article_update_view(request, id):
    article = get_object_or_404(Article, pk=id)
    if request.method == 'GET':
        form = ArticleForm(initial={
            'title': article.title,
            'status': article.status,
            'author': article.author,
            'text': article.text
        })
        return render(request, 'update.html', context={'article': article, 'form': form, 'status_choices': StatusChoices.choices})
    elif request.method == 'POST':
        form = ArticleForm(data=request.POST)
        if form.is_valid():

            article.title = request.POST.get('title')
            article.author = request.POST.get('author')
            article.text = request.POST.get('text')
            article.status = request.POST.get('status')
            article.save()
            return redirect('articles-detail', id=article.pk)
        else:
            return render(request, 'update.html', context={'article': article, 'form': form, 'status_choices': StatusChoices.choices})
    

def delete_article(request, id):
    article = get_object_or_404(Article, pk=id)
    if request.method == 'GET':
        return render(request, 'delete.html', context={'article': article})

    elif request.method == 'POST':
        article.delete()
        return redirect('main_page')
